==> xh_framework-0.0.1.tar.gz: lib/com/xh/refresh/xh_refresh_state.dart <==
import 'dart:async';
import 'package:flutter/material.dart';
import 'package:xh_framework/com/xh/base/xh_list_widget.dart';

abstract class XHRefreshStatefulWidgetState<T extends XHListStatefulWidget,
    Model> extends XHListStatefulWidgetState<T, Model> {
  int pageIndex = 1;
  String message = '';
  bool _showProgress = false;
  List<Model> items = new List<Model>();
  ScrollController scrollController = new ScrollController();

  Widget buildList(BuildContext context, int index);

  @override
  void initState() {
    super.initState();
    _showProgress = true;
    onRefresh(pageIndex);
  }

  @override
  void dispose() {
    scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (_showProgress) {
      _showProgress = false;
      return Center(
        child: new CircularProgressIndicator(),
      );
    }

    return new RefreshIndicator(
      onRefresh: _onRefresh,
      child: new Stack(
        children: <Widget>[_buildMessageWidget(), buildListWidget()],
      ),
    );
  }

  Future<void> onRefresh(int pageIndex) {
    return null;
  }

  void onRefreshComplete(List<Model> list) {
    setState(() {
      message = list.isEmpty ? '暂无数据' : '';
      items.clear();
      items.addAll(list);
    });
  }

  void onRefreshError(Exception e) {
    setState(() {
      message = e.toString();
    });
  }

  Future<void> _onRefresh() {
    pageIndex = 1;
    return onRefresh(pageIndex);
  }

  Widget buildListWidget() {
    return new Offstage(
      offstage: message.isNotEmpty,
      child: ListView.builder(
          controller: scrollController,
          itemCount: items.length,
          itemBuilder: (context, index) {
            return buildList(context, index);
          }),
    );
  }

  Widget buildLoadMoreIndicator() {
    return null;
  }

  Widget _buildMessageWidget() {
    return new Offstage(
        offstage: message.isEmpty,
        child: new ListView.builder(
            itemCount: 1,
            itemBuilder: (context, index) {
              return new GestureDetector(
                child: new Container(
                  padding: const EdgeInsets.only(top: 200.0),
                  child: new Text(
                    message,
                    textAlign: TextAlign.center,
                    style: TextStyle(
                        fontSize: 16.0,
                        decoration: TextDecoration.none,
                        color: Colors.black38),
                  ),
                ),
              );
            }));
  }

  Model getItem(int index) {
    if (items.isNotEmpty) {
      return items.elementAt(index);
    }
    return null;
  }
}

==> xh_framework-0.0.3.tar.gz: lib/com/xh/refresh/xh_refresh_state.dart <==
import 'dart:async';
import 'package:flutter/material.dart';
import 'package:xh_framework/com/xh/base/xh_list_widget.dart';
import 'package:xh_framework/xh_framework.dart';

abstract class XHRefreshStatefulWidgetState<T extends XHListStatefulWidget,
    Model> extends XHListStatefulWidgetState<T, Model> {
  int pageIndex = 1;
  String message = '';
  bool _showProgress = false;
  List<Model> items = new List<Model>();
  ScrollController scrollController = new ScrollController();
  Completer completer = new Completer();

  Widget buildList(BuildContext context, int index);

  @override
  void initState() {
    super.initState();
    _showProgress = true;
    onRefresh(pageIndex);
  }

  @override
  void dispose() {
    scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (_showProgress) {
      _showProgress = false;
      return Center(
        child: new CircularProgressIndicator(),
      );
    }

    return new RefreshIndicator(
      onRefresh: _onRefresh,
      child: new Stack(
        children: <Widget>[_buildMessageWidget(), buildListWidget()],
      ),
    );
  }

  void onRefresh(int pageIndex) {

  }

  void onRefreshComplete(List<Model> list) {
    completer.complete();
    setState(() {
      message = list.isEmpty ? '暂无数据' : '';
      items.clear();
      items.addAll(list);
    });
  }

  void onRefreshError(Exception e) {
    if (!completer.isCompleted) {
      completer.complete();
    }
    setState(() {
      message = (e is NetworkException) ? e.message : e.toString();
    });
  }

  Future<void> _onRefresh() {
    completer = new Completer();
    pageIndex = 1;
    onRefresh(pageIndex);
    return completer.future;
  }

  Widget buildListWidget() {
    return new Offstage(
      offstage: message.isNotEmpty,
      child: ListView.builder(
          controller: scrollController,
          itemCount: items.length,
          itemBuilder: (context, index) {
            return buildList(context, index);
          }),
    );
  }

  Widget buildLoadMoreIndicator() {
    return null;
  }

  Widget _buildMessageWidget() {
    return new Offstage(
        offstage: message.isEmpty,
        child: new ListView.builder(
            itemCount: 1,
            itemBuilder: (context, index) {
              return new GestureDetector(
                child: new Container(
                  padding: const EdgeInsets.only(top: 200.0),
                  child: new Text(
                    message,
                    textAlign: TextAlign.center,
                    style: TextStyle(
                        fontSize: 16.0,
                        decoration: TextDecoration.none,
                        color: Colors.black38),
                  ),
                ),
              );
            }));
  }

  Model getItem(int index) {
    if (items.isNotEmpty) {
      return items.elementAt(index);
    }
    return null;
  }
}
